
print('KMMTransferReg : a transfer learning regression model based on Kernel Mean Matching (KMM) algorithm \n Bin CAO, ZheJiang LAB, Hangzhou, CHINA. (MGI, SHU, Shanghai, CHINA).')

__version__ = '1.0'
__description__ = 'a transfer learning regression model'
__author__ = 'Bin CAO'
__author_email__ = 'binjacobcao@gmail.com'
__url__ = 'https://github.com/Bin-Cao/KMMTransferRegressor'
